package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.bean.dto.BlurryUserDto;
import com.qmth.distributed.print.business.bean.dto.LoginDto;
import com.qmth.distributed.print.business.bean.dto.UserDto;
import com.qmth.distributed.print.business.entity.*;
import com.qmth.distributed.print.business.enums.RoleTypeEnum;
import com.qmth.distributed.print.business.mapper.SysUserMapper;
import com.qmth.distributed.print.business.service.*;
import com.qmth.distributed.print.business.util.ServletUtil;
import com.qmth.distributed.print.business.util.security.Md5Utils;
import com.qmth.distributed.print.common.contant.SystemConstant;
import com.qmth.distributed.print.common.enums.ExceptionResultEnum;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import sun.misc.BASE64Encoder;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 用户表 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class SysUserServiceImpl extends ServiceImpl<SysUserMapper, SysUser> implements SysUserService {

    @Autowired
    private SysRoleService sysRoleService;

    @Autowired
    private BasicCourseService basicCourseService;

    @Autowired
    private BasicUserCourseService basicUserCourseService;

    @Autowired
    private SysConfigService sysConfigService;

    @Autowired
    private SysUserRoleService sysUserRoleService;

    @Autowired
    private SysRolePrivilegeService sysRolePrivilegeService;

    @Resource
    SysUserMapper sysUserMapper;

    @Autowired
    CacheService cacheService;

    @Autowired
    private CommonService commonService;

    @Override
    public IPage<UserDto> list(String realName, String roleId, Boolean enable, Integer pageNumber, Integer pageSize) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());

        Page<UserDto> page = new Page<>(pageNumber, pageSize);
        IPage<UserDto> userDtoIPage = this.baseMapper.listPage(page, schoolId, realName, roleId, enable);
        if (userDtoIPage.getRecords().size() > 0) {
            userDtoIPage.getRecords().forEach(m -> {
                //角色
                List<SysRole> roles = sysRoleService.listRolesByUserId(Long.valueOf(m.getId()));
                m.setRoles(roles);

                //课程
                List<BasicCourse> courses = basicCourseService.listCoursesByUserId(Long.valueOf(m.getId()));
                m.setCourses(courses);
            });
        }
        return userDtoIPage;
    }


    @Transactional
    @Override
    public boolean saveUser(SysUser user) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        user.setSchoolId(schoolId);
        SysUser sysUser1 = (SysUser) ServletUtil.getRequestUser();

        QueryWrapper<SysUser> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysUser::getLoginName, user.getLoginName());
        SysUser sysUser = this.getOne(queryWrapper);
        boolean isSuccess;
        // 新增
        if (Objects.isNull(user.getId())) {
            if (sysUser != null) {
                throw ExceptionResultEnum.ERROR.exception("用户名已存在");
            }
            SysConfig sysConfig = sysConfigService.getByKey("sys.user.initPassword");
            BASE64Encoder encoder = new BASE64Encoder();
            user.setPassword(encoder.encode(StringUtils.isNoneBlank(sysConfig.getConfigValue()) ? sysConfig.getConfigValue().getBytes() : "123456".getBytes()));
            user.setId(SystemConstant.getDbUuid());
            user.setCreateId(sysUser1.getId());
            user.setCreateTime(System.currentTimeMillis());
            isSuccess = this.save(user);
        }
        // 修改
        else {
            if (sysUser != null && user.getId().longValue() != sysUser.getId().longValue()) {
                throw ExceptionResultEnum.ERROR.exception("用户名已存在");
            }
            sysUser.setRealName(user.getRealName());
            sysUser.setMobileNumber(user.getMobileNumber());
            sysUser.setOrgId(user.getOrgId());
            sysUser.setUpdateId(sysUser1.getId());
            sysUser.setUpdateTime(System.currentTimeMillis());
            isSuccess = this.updateById(sysUser);

            //删除角色
            UpdateWrapper<SysUserRole> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda().eq(SysUserRole::getUserId, sysUser.getId());

            //删除课程
            basicUserCourseService.removeByUserId(sysUser.getId());

            sysUserRoleService.remove(updateWrapper);
        }

        //新增用户-角色
        for (Long roleId : user.getRoleIds()) {
            List<SysRolePrivilege> rolePrivileges = sysRolePrivilegeService.listByRoleId(roleId);
            for (SysRolePrivilege rolePrivilege : rolePrivileges) {
                SysUserRole userRole = new SysUserRole();
                userRole.setUserId(user.getId());
                userRole.setRoleId(roleId);
                userRole.setPrivilegeId(rolePrivilege.getPrivilegeId());
                sysUserRoleService.save(userRole);
            }
            cacheService.removeRolePrivilegeCache(roleId);
        }

        // 角色里是否有考务老师角色
        List<SysRole> sysRoles = sysRoleService.list(user.getRoleIds(), RoleTypeEnum.QUESTION_TEACHER.name());
        if (sysRoles != null && sysRoles.size() > 0) {
            Long[] courseIds = user.getCourseIds();
            if (courseIds.length == 0) {
                throw ExceptionResultEnum.ERROR.exception("请选择课程");
            }

            basicUserCourseService.saveBatch(user);

        }

        // 清除缓存
        cacheService.removeUserAuthCache(user.getId());

        return isSuccess;
    }

    @Override
    public boolean enable(SysUser user) {
        UpdateWrapper<SysUser> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(SysUser::getEnable, user.getEnable()).eq(SysUser::getId, user.getId());
        return this.update(updateWrapper);
    }

    @Override
    public boolean resetPassword(Long id) {
        SysConfig sysConfig = sysConfigService.getByKey("sys.user.initPassword");
        BASE64Encoder encoder = new BASE64Encoder();
        String md5Password = encoder.encode(StringUtils.isNoneBlank(sysConfig.getConfigValue()) ? sysConfig.getConfigValue().getBytes() : "123456".getBytes());

        SysUser user = this.getById(id);
        user.setPassword(md5Password);

        return this.updateById(user);
    }

    @Override
    public boolean updatePassword(SysUser user) {
        SysUser sysUser = this.getById(user.getId());
        if (sysUser == null) {
            throw ExceptionResultEnum.ERROR.exception("用户不存在");
        }
        if (!StringUtils.equals(sysUser.getPassword(), user.getOldPassword())) {
            throw ExceptionResultEnum.ERROR.exception("原密码不正确");
        }
        sysUser.setPassword(user.getPassword());
        return this.updateById(sysUser);
    }

    @Transactional
    @Override
    public boolean bindRoles(SysUser sysUser) {
        Long[] roleIds = sysUser.getRoleIds();
        if (roleIds.length == 0) {
            throw ExceptionResultEnum.ERROR.exception("请选择角色");
        }

        // 删除
        sysUserRoleService.removeByUserId(sysUser.getId());

        List<SysUserRole> list = new ArrayList<>();
        for (Long roleId : roleIds) {
            List<SysRolePrivilege> rolePrivileges = sysRolePrivilegeService.listByRoleId(roleId);
            for (SysRolePrivilege rolePrivilege : rolePrivileges) {
                SysUserRole userRole = new SysUserRole();
                userRole.setUserId(sysUser.getId());
                userRole.setRoleId(roleId);
                userRole.setPrivilegeId(rolePrivilege.getPrivilegeId());
                list.add(userRole);
            }
        }

        // 角色里是否有考务老师角色
        List<SysRole> sysRoles = sysRoleService.list(roleIds, RoleTypeEnum.QUESTION_TEACHER.name());
        if (sysRoles != null && sysRoles.size() > 0) {
            Long[] courseIds = sysUser.getCourseIds();
            if (courseIds.length == 0) {
                throw ExceptionResultEnum.ERROR.exception("请选择课程");
            }

            basicUserCourseService.removeByUserId(sysUser.getId());
            basicUserCourseService.saveBatch(sysUser);

        }
        return sysUserRoleService.saveBatch(list);
    }

    @Override
    public LoginDto getUserRoles(SysUser sysUser) {
        LoginDto loginDto = new LoginDto();
        loginDto.setId(String.valueOf(sysUser.getId()));
        loginDto.setSchoolId(String.valueOf(sysUser.getSchoolId()));
        loginDto.setLoginName(sysUser.getLoginName());
        loginDto.setRealName(sysUser.getRealName());
        loginDto.setOrgId(String.valueOf(sysUser.getOrgId()));
        loginDto.setEnable(sysUser.getEnable());
        loginDto.setPwdUpdateTime(sysUser.getPwdUpdateTime());

        if ("admin".equals(sysUser.getLoginName())) {
            loginDto.setRoles(Arrays.asList("ADMIN"));
        } else {
            List<SysRole> list = sysUserRoleService.listRoleByUserId(sysUser.getId());
            List<String> roles = list.stream().map(m -> m.getType().name()).collect(Collectors.toList());
            loginDto.setRoles(roles);
        }
        return loginDto;
    }

    @Override
    public List<BlurryUserDto> listUser(String courseCode, String param) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        Set<Long> orgIds = commonService.listSubOrgIds();
        return this.baseMapper.listUser(schoolId, RoleTypeEnum.QUESTION_TEACHER.name(), courseCode, param, orgIds);
    }

    @Override
    public List<SysUser> listByOrgId(Long id) {
        QueryWrapper<SysUser> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysUser::getOrgId, id);
        return this.list(queryWrapper);
    }

    @Transactional
    @Override
    public boolean saveCustomer(SysUser user) {
        SysUser sysUser1 = (SysUser) ServletUtil.getRequestUser();

        QueryWrapper<SysUser> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysUser::getLoginName, user.getLoginName());
        SysUser sysUser = this.getOne(queryWrapper);
        boolean isSuccess;
        // 新增
        if (Objects.isNull(user.getId())) {
            if (sysUser != null) {
                throw ExceptionResultEnum.ERROR.exception("用户名已存在");
            }
            SysConfig sysConfig = sysConfigService.getByKey("sys.user.initPassword");
            BASE64Encoder encoder = new BASE64Encoder();
            user.setPassword(encoder.encode(StringUtils.isNoneBlank(sysConfig.getConfigValue()) ? sysConfig.getConfigValue().getBytes() : "123456".getBytes()));
            user.setId(SystemConstant.getDbUuid());
            user.setCreateId(sysUser1.getId());
            user.setCreateTime(System.currentTimeMillis());
            isSuccess = this.save(user);
        }
        // 修改
        else {
            if (sysUser != null && user.getId().longValue() != sysUser.getId().longValue()) {
                throw ExceptionResultEnum.ERROR.exception("用户名已存在");
            }
            sysUser.setRealName(user.getRealName());
            sysUser.setMobileNumber(user.getMobileNumber());
            sysUser.setUpdateId(sysUser1.getId());
            sysUser.setUpdateTime(System.currentTimeMillis());
            isSuccess = this.updateById(sysUser);

            //删除角色
            UpdateWrapper<SysUserRole> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda().eq(SysUserRole::getUserId, sysUser.getId());
            sysUserRoleService.remove(updateWrapper);
        }

        //新增用户-角色
        for (Long roleId : user.getRoleIds()) {
            SysRole sysRole = sysRoleService.getById(roleId);
            if(!RoleTypeEnum.CUSTOMER.name().equals(sysRole.getType().name())){
                throw ExceptionResultEnum.ERROR.exception("只能添加客服角色的用户");
            }
            List<SysRolePrivilege> rolePrivileges = sysRolePrivilegeService.listByRoleId(roleId);
            for (SysRolePrivilege rolePrivilege : rolePrivileges) {
                SysUserRole userRole = new SysUserRole();
                userRole.setUserId(user.getId());
                userRole.setRoleId(roleId);
                userRole.setPrivilegeId(rolePrivilege.getPrivilegeId());
                sysUserRoleService.save(userRole);
            }
            cacheService.removeRolePrivilegeCache(roleId);
        }

        // 清除缓存
        cacheService.removeUserAuthCache(user.getId());

        return isSuccess;
    }

    @Override
    public IPage<UserDto> listCustomer(String realName, Boolean enable, Integer pageNumber, Integer pageSize) {
        Page<UserDto> page = new Page<>(pageNumber, pageSize);
        IPage<UserDto> userDtoIPage = this.baseMapper.listCustomerPage(page, realName, enable);
        if (userDtoIPage.getRecords().size() > 0) {
            userDtoIPage.getRecords().forEach(m -> {
                //角色
                List<SysRole> roles = sysRoleService.listRolesByUserId(Long.valueOf(m.getId()));
                m.setRoles(roles);

                //课程
                List<BasicCourse> courses = basicCourseService.listCoursesByUserId(Long.valueOf(m.getId()));
                m.setCourses(courses);
            });
        }
        return userDtoIPage;
    }
}
